#include "lyngoo.h"
#include <new>
using namespace std;

LynGOOConf config;
bool keepRunning = true;


// **********************************************************
void LynGOODebug()
{
   char str[100];
   sprintf(str,"%d%% %d %d",mEmulationSpeed,mFramesPerSecond,mFrameSkip);
   PgPrint(0,0,RED,str,0);
}

// **********************************************************
UBYTE* LynGOODisplayCallback(ULONG objref) {
   static int frameskipcount = 0;

   PgSetVisible(config.screenWidth,config.screenHeight);

   if (frameskipcount>0) {
      frameskipcount--;
   } else {
      frameskipcount=mFrameSkip;
      mFrameCount++;
//      LynGOODebug();
      PgScreenFlip();
      ClearDoubleBuffer();
   }

   UBYTE* VramAddr = (UBYTE*)PgGetVramAddr((config.screenWidth-HANDY_SCREEN_WIDTH)/2,(config.screenHeight-HANDY_SCREEN_HEIGHT)/2);

   PgSetVisible(DINGOO_SCREEN_WIDTH,DINGOO_SCREEN_HEIGHT);

   return VramAddr;
}

// **********************************************************
void LynGOOSetup()
{
   switch (config.screenMode) {
      case SCREENMODE_1X:
         config.screenWidth=DINGOO_SCREEN_WIDTH;
         config.screenHeight=DINGOO_SCREEN_HEIGHT;
         config.lynxSystem->DisplaySetAttributes(MIKIE_NO_ROTATE,MIKIE_PIXEL_FORMAT_16BPP_565,DINGOO_SCREEN_WIDTH*sizeof(uint16_t),LynGOODisplayCallback,NULL);
      break;
      case SCREENMODE_2X:
         config.screenWidth=HANDY_SCREEN_WIDTH;
         config.screenHeight=DINGOO_SCREEN_HEIGHT/2;
         config.lynxSystem->DisplaySetAttributes(MIKIE_NO_ROTATE,MIKIE_PIXEL_FORMAT_16BPP_565,HANDY_SCREEN_WIDTH*sizeof(uint16_t),LynGOODisplayCallback,NULL);
      break;
      case SCREENMODE_STRETCH:
      default:
         config.screenWidth=HANDY_SCREEN_WIDTH;
         config.screenHeight=HANDY_SCREEN_HEIGHT;
         config.lynxSystem->DisplaySetAttributes(MIKIE_NO_ROTATE,MIKIE_PIXEL_FORMAT_16BPP_565,HANDY_SCREEN_WIDTH*sizeof(uint16_t),LynGOODisplayCallback,NULL);
      break;
   }
   gAudioEnabled = true;
}

// **********************************************************
void LynGOOInit(int argc, char *argv[])
{
   // initialize pad
   PadInit();

   // initialize graphics
   PgInit();
   ClearDoubleBuffer();
   PgScreenFlip();

   // initialize config
   config.loopCount = 1000;
   config.screenMode = SCREENMODE_2X;
   config.sndLevel = 30;
   config.lynxSystem = NULL;
   memset(config.selectedRomName, 0, MAXPATH);
   config.lynxKeyUp = DK_DPAD_UP;
   config.lynxKeyDown = DK_DPAD_DOWN;
   config.lynxKeyLeft = DK_DPAD_LEFT;
   config.lynxKeyRight = DK_DPAD_RIGHT;
   config.lynxKeyA = DK_BUTTON_A;
   config.lynxKeyB = DK_BUTTON_B;
   config.lynxKeyOPT1 = DK_BUTTON_X;
   config.lynxKeyOPT2 = DK_BUTTON_Y;
   config.lynxKeyStart = DK_BUTTON_START;

   // initialize selected rom name
   if (argc>0) strcpy(config.selectedRomName,argv[0]);

   // initialize lynx
   config.lynxSystem = new CSystem(config.selectedRomName, ROM_FILE);
   if (config.lynxSystem == NULL) {
      Error("Error creating lynx",1);
      keepRunning = false;
   } else {
      LynGOOSetup();
   }

   // initialize sound
   if (!SoundInit()) {
      Error("Error initializing sound thread",1);
      keepRunning = false;
   }

   // initialize throttle thread
   if (!ThrottleInit()) {
      Error("Error initializing throttle thread",1);
      keepRunning = false;
   }
}

// **********************************************************
void LynGOOExit()
{
   ThrottleExit();
   SoundExit();
   delete config.lynxSystem;
   PgExit();
   PadExit();
}

// **********************************************************
uint32_t LynGOOMenu(uint32_t menu)
{
   uint32_t result;

   switch (menu)
   {
   case ACTIONS_VIEW:
   default:
      result = ActionSelectView();
      break;
   }

   if (result & VIEW_REQUEST)
   {
      LynGOOMenu(result ^= VIEW_REQUEST);
   }
   else if (result & (ACTIONS_VIEW))
   {
      uint32_t action = (result ^= ACTIONS_VIEW);

      switch (action)
      {
      case ACTION_LOAD_STATE:
         if (Question("Load state?")==DK_BUTTON_A) 
           LoadState();
         break;
      case ACTION_SAVE_STATE:
         if (Question("Save state?")==DK_BUTTON_A) 
           SaveState();
         break;
      case ACTION_RESET:
         if (config.lynxSystem != NULL)
            config.lynxSystem->Reset();
         break;
      case ACTION_QUIT:
         keepRunning = false;
         break;
      case ACTION_CONTINUE:
         ClearDoubleBuffer();
         break;
      }
   }
   return 0;
}

// **********************************************************
void SaveState()
{
   char contextName[MAXPATH];
   char romName[MAXPATH];

   if (config.lynxSystem != NULL)
   {
      strncpy(romName, config.selectedRomName, strlen(config.selectedRomName) - 4);
      romName[strlen(config.selectedRomName) - 4] = '\0';
      sprintf(contextName, "%s%s", romName, STA_EXT);
      if (!(config.lynxSystem->ContextSave(contextName)))
         Error("Error saving state file.", 1);
   }
}

// **********************************************************
void LoadState()
{
   char contextName[MAXPATH];
   char romName[MAXPATH];

   if (config.lynxSystem != NULL)
   {
      strncpy(romName, config.selectedRomName, strlen(config.selectedRomName) - 4);
      romName[strlen(config.selectedRomName) - 4] = '\0';
      sprintf(contextName, "%s%s", romName, STA_EXT);
      if (!(config.lynxSystem->ContextLoad(contextName)))
         Error("Error loading state file.", 1);
   }
}

// **********************************************************
int GetKeyIndex(int key)
{
   switch (key)
   {
   case DK_DPAD_UP:
      return 0;
   case DK_DPAD_DOWN:
      return 1;
   case DK_DPAD_LEFT:
      return 2;
   case DK_DPAD_RIGHT:
      return 3;
   case DK_BUTTON_A:
      return 4;
   case DK_BUTTON_B:
      return 5;
   case DK_BUTTON_X:
      return 6;
   case DK_BUTTON_Y:
      return 7;
   case DK_BUTTON_START:
      return 8;
   case DK_BUTTON_SELECT:
      return 9;
   case DK_TRIGGER_RIGHT:
      return 10;
   case DK_TRIGGER_LEFT:
      return 11;
   }

   return 0;
}

// **********************************************************
void MakeActionListView(const char **actionList, int numItems, int cursor)
{
   char buf[15];
   unsigned long color;

   ClearDoubleBuffer();
   PgPrint(14, 1, WHITE, FILE_VERSION, 0);
   for (int i = 0, j = 6; i < numItems; i++, j++)
   {
      if (i == cursor) color=RED; else color=WHITE;
      PgPrint(2, j, color, actionList[i], 0);
      switch(i+1) {
         case ACTION_VOLUME:
            sprintf(buf, "%d%%", config.sndLevel);
            PgPrint(14, j, color, buf, 0);
            break;
         case ACTION_SCREENMODE:
            switch (config.screenMode) {
               case SCREENMODE_1X:
                  sprintf(buf, " 1x");
               break;
               case SCREENMODE_2X:
                  sprintf(buf, " 2x");
               break;
               case SCREENMODE_STRETCH:
               default:
                  sprintf(buf, " 2x (Stretched)");
               break;
            }
            PgPrint(14, j, color, buf, 0);
            break;
      }
   }
   PgScreenFlip();
}

// **********************************************************
uint32_t ActionSelectView()
{
   int cursor = 0;
   const char *actionList[7] = { "Continue", "Volume", "Scaling", "Load state", "Save state", "Reset", "Quit" };
   int numItems = 7;
   char selected = -1;
   char action = 0;

   while (selected < 0 && action < 2)
   {
      MakeActionListView(actionList, numItems, cursor);

      action = 0;

      while (action == 0)
      {
         PadKeyTimer();
         switch (PadWaitForOneKey())
         {
         case DK_DPAD_UP:
            cursor--;
            action = 1;
            if (cursor < 0)
               cursor = (numItems-1);
            break;
         case DK_DPAD_DOWN:
            cursor++;
            action = 1;
            if (cursor >= numItems)
               cursor = 0;
            break;
         case DK_DPAD_LEFT:
            action = 1;
            switch (cursor+1) {
               case ACTION_VOLUME:
                  config.sndLevel = config.sndLevel <= 0 ? 0 : config.sndLevel - 5;
                  break;
               case ACTION_SCREENMODE:
                  config.screenMode = config.screenMode <= SCREENMODE_1X ? SCREENMODE_1X : config.screenMode - 1;
                  break;
            }
            break;
         case DK_DPAD_RIGHT:
            action = 1;
            switch (cursor+1) {
               case ACTION_VOLUME:
                  config.sndLevel = config.sndLevel >= 100 ? 100 : config.sndLevel + 5;
                  break;
               case ACTION_SCREENMODE:
                  config.screenMode = config.screenMode >= SCREENMODE_STRETCH ? SCREENMODE_STRETCH : config.screenMode + 1;
                  break;
            }
            break;
         case DK_BUTTON_A:
            switch (cursor+1) {
               case ACTION_VOLUME:
               case ACTION_SCREENMODE:
                  break;
               default:
                  selected = cursor + 1;
                  action = 1;
                  break;
            }
            break;
         case DK_BUTTON_B:
            selected = ACTION_CONTINUE;
            action = 1;
            break;
         default:
            break;
         }
      }
   }

   return (ACTIONS_VIEW | selected);
}

// **********************************************************
int main(int argc, char *argv[])
{
   char pause = 0;
   int volume = 0;
   ULONG mask = 0;

   // initialize
   LynGOOInit(argc,argv);

   // loop
   while (keepRunning)
   {
      // poll controller
      control_poll();
      mask = 0;

      // check lynx keys
      if (control_pressed(config.lynxKeyUp))
         mask |= BUTTON_UP;
      else if (control_pressed(config.lynxKeyDown))
         mask |= BUTTON_DOWN;
      if (control_pressed(config.lynxKeyLeft))
         mask |= BUTTON_LEFT;
      else if (control_pressed(config.lynxKeyRight))
         mask |= BUTTON_RIGHT;
      if (control_pressed(config.lynxKeyB))
         mask |= BUTTON_B;
      if (control_pressed(config.lynxKeyA))
         mask |= BUTTON_A;
      if (control_pressed(config.lynxKeyOPT2))
         mask |= BUTTON_OPT2;
      if (control_pressed(config.lynxKeyOPT1))
         mask |= BUTTON_OPT1;
      if (control_pressed(config.lynxKeyStart))
         mask |= BUTTON_PAUSE;

      // check menu
      if (control_pressed(DK_BUTTON_SELECT) && control_pressed(DK_BUTTON_START)) {
         pause = 1;
         mask = 0;
      }

      // check volume up
      if (control_pressed(DK_TRIGGER_RIGHT) && control_changed(DK_TRIGGER_RIGHT)) {
         config.sndLevel = config.sndLevel >= 100 ? 100 : config.sndLevel + 5;
      }

      // check volume down
      if (control_pressed(DK_TRIGGER_LEFT) && control_changed(DK_TRIGGER_LEFT)) {
         config.sndLevel = config.sndLevel <= 0 ? 0 : config.sndLevel - 5;
      }

      // pass lynx keys
      config.lynxSystem->SetButtonData(mask);

      // set volume if needed
      if (volume!=config.sndLevel) {
         volume=config.sndLevel;
         waveout_set_volume(volume>>1);
      }

      // handle menu
      if (pause == 1)
      {
         gSystemHalt = 1;
         SoundPause();
         LynGOOMenu(ACTIONS_VIEW);
         if (keepRunning) {
            ClearDoubleBuffer();
            LynGOOSetup();
            pause = 0;
            gSystemHalt = 0;
            SoundUnPause();
         }
      }

      // update lynx
      if (ThrottleCheck() && !gSystemHalt) config.lynxSystem->Update(config.loopCount);
   }

   // exit
   LynGOOExit();

   return 0;
}
